import { Component, Input, OnInit } from "@angular/core";

import { SelectField } from "../../formulaire/select-field";
import { SelectEntry } from "../../formulaire/select-entry";
import { I18nService } from "../../services/internationalisation.service";
import { SelectFieldReference } from "../../formulaire/select-field-reference";

@Component({
    selector: "select-field-line",
    templateUrl: "../generic-select/generic-select.component.html",
    styleUrls: [
        "./select-field-line.component.scss"
    ]
})
export class SelectFieldLineComponent implements OnInit {

    /** aide en ligne */
    protected helpLink: string | { [key: string]: string };

    @Input()
    private _select: SelectField;

    public constructor(private i18nService: I18nService) {}

    public get selectId() {
        return this._select.id;
    }

    public get entries(): SelectEntry[] {
        if (! this._select) {
            return [];
        }
        return this._select.entries;
    }

    protected entryLabel(entry: SelectEntry): string {
        return entry.label;
    }

    public get selectedValue(): SelectEntry {
        return this._select.getValue();
    }

    public get isValid(): boolean {
        return (this._select.getValue() !== undefined);
    }

    public set selectedValue(v: SelectEntry) {
        this._select.setValue(v);
    }

    public get label() {
        if (this._select) {
            return this._select.label;
        } else {
            return "";
        }
    }

    public openHelp($event: any) {
        let link;
        if (typeof this._select.helpLink === "string") {
            link = this._select.helpLink;
        } else { // object
            const cv = this._select.getValue();
            const entryId = cv.id.substring(this._select.entriesBaseId.length);
            link = this._select.helpLink[entryId];
        }
        window.open("assets/docs-fr/calculators/" + link, "_blank");
        $event.preventDefault();
        $event.stopPropagation();
        return false;
    }

    public get enableHelpButton() {
        if (this._select && this._select.helpLink) {
            if (typeof this._select.helpLink === "string") {
                return true;
            } else { // object
                const cv = this._select.getValue();
                const entryId = cv.id.substring(this._select.entriesBaseId.length);
                return Object.keys(this._select.helpLink).includes(entryId);
            }
        }
        return false;
    }

    public get uitextOpenHelp() {
        return this.i18nService.localizeText("INFO_CALCULATOR_OPEN_HELP");
    }

    // called every time we navigate to the module
    ngOnInit(): void {
        if (this._select instanceof SelectFieldReference) {
            this._select.updateEntries();
        }
    }
}
