import { FormulaireDefinition } from "../form-definition";
import { CalculatorResults } from "../../../results/calculator-results";
import { FormResult } from "../form-result";
import { FormCompute } from "../form-compute";
import { FormResultFixedVar } from "../form-result-fixedvar";
import { FormComputeFixedVar } from "../form-compute-fixedvar";
import { ServiceFactory } from "../../../services/service-factory";

export class FormulaireBase extends FormulaireDefinition {

    protected _formCompute: FormCompute;

    protected _formResult: FormResult;

    constructor() {
        super();
        this._formResult = new FormResultFixedVar(this);
        this._formCompute = new FormComputeFixedVar(this, (this._formResult as FormResultFixedVar));
    }

    protected completeParse(json: {}) {
        super.completeParse(json);
        this._formResult.helpLinks = this._resultsHelpLinks;
    }

    /**
     * Resets the form results, the results panel on screen, the model
     * results, and does the same for all depending modules
     * @param symbol symbol of the parameter whose value change triggered this method
     * @param forceResetAllDependencies if true, even non-calculated non-modified parameters
     *      links will be considered as dependencies @see jalhyd#98
     */
    public resetResults(visited: string[] = [], symbol?: string, forceResetAllDependencies: boolean = false) {
        visited.push(this.currentNub.uid);
        // reset GUI results
        this._formResult.resetResults();
        // reset model results
        this.currentNub.resetResult();
        // reset the result panels of all forms depending on this one
        ServiceFactory.instance.formulaireService.resetAllDependingFormsResults(this, visited, symbol, forceResetAllDependencies);
    }

    public doCompute() {
        this._formCompute.doCompute();
    }

    public get hasResults(): boolean {
        return this._formResult.hasResults;
    }

    public get results(): CalculatorResults[] {
        return this._formResult.results;
    }
}
