import { IObservable, ParamDefinition, Solveur } from "jalhyd";

import { FormulaireBase } from "./form-base";
import { SelectFieldNub } from "../../select-field-nub";
import { SelectFieldParameter } from "../../select-field-parameter";
import { NgParameter } from "../../ngparam";

/**
 * Formulaire pour les Solveurs
 */
export class FormulaireSolveur extends FormulaireBase {

    /** id of select configuring target Nub */
    private _targetNubSelectId: string;

    /** id of select configuring searched param */
    private _searchedParamSelectId: string;

    protected parseOptions(json: {}) {
        super.parseOptions(json);
        this._targetNubSelectId = this.getOption(json, "targetNubSelectId");
        this._searchedParamSelectId = this.getOption(json, "searchedParamSelectId");
    }

    protected completeParse(json: {}) {
        super.completeParse(json);
        if (this._targetNubSelectId) {
            const sel = this.getFormulaireNodeById(this._targetNubSelectId);
            if (sel) {
                sel.addObserver(this);
                // force 1st observation
                (sel as SelectFieldNub).notifySelectValueChanged();
            }
        }
        if (this._searchedParamSelectId) {
            const sel = this.getFormulaireNodeById(this._searchedParamSelectId);
            if (sel) {
                sel.addObserver(this);
                // force 1st observation
                (sel as SelectFieldNub).notifySelectValueChanged();
            }
        }

    }

    private debugState() {
        const sol = this._currentNub as Solveur;
        const spm = sol.searchedParameter;
        console.log(
            `ETAT:\n X.singleValue=${sol.prms.X ? sol.prms.X.singleValue : "UNDEF"}\n Y.singleValue=${sol.prms.Y.singleValue}`
            + `\n Xinit.singleValue=${sol.prms.Xinit.singleValue}\n Ytarget.singleValue=${sol.prms.Ytarget.singleValue}`
            + `\n searchedParam.singleValue=${spm ? spm.singleValue : "UNDEF"}`
        );
    }

    // interface Observer

    public update(sender: IObservable, data: any) {
        super.update(sender, data);
        if (sender instanceof SelectFieldNub) {
            if (data.action === "select") {
                // update Solveur property: Nub to calculate
                try {
                    // if searchedParam is set to a value that won't be available anymore
                    // once nubToCalculate is updated, setPropValue throws an error, but
                    // nubToCalculate is updated anyway; here, just inhibit the error
                    this._currentNub.properties.setPropValue("nubToCalculate", data.value.value);
                } catch (e) { }
                // refresh parameters selector
                const sel = this.getFormulaireNodeById(this._searchedParamSelectId) as SelectFieldParameter;
                if (sel) {
                    sel.updateEntries();
                    // reflect changes in GUI
                    const inputYtarget = this.getFormulaireNodeById("Ytarget") as NgParameter;
                    inputYtarget.notifyValueModified(this);
                }
            }
        }
        if (sender instanceof SelectFieldParameter) {
            if (data.action === "select") {
                // update Solveur property: searched Parameter
                try {
                    const p: ParamDefinition = data.value.value;
                    this._currentNub.properties.setPropValue(
                        "searchedParameter",
                        p.nubUid + "/" + p.symbol
                    );
                } catch (e) { }
                // reflect changes in GUI
                const inputXinit = this.getFormulaireNodeById("Xinit") as NgParameter;
                inputXinit.notifyValueModified(this);
            }
        }
    }
}
