import { Nub, Result, ParamDomainValue, Observer, ParamDefinition } from "jalhyd";

import { FormResult } from "./form-result";
import { FormulaireDefinition } from "./form-definition";
import { NgParameter, ParamRadioConfig } from "../ngparam";

export abstract class FormCompute implements Observer {

    constructor(protected _formBase: FormulaireDefinition, protected _formResult: FormResult) {
        // indirectly subscribe to Nub result updates
        this._formBase.addObserver(this);
    }

    protected abstract compute();

    protected get formResult(): FormResult {
        return this._formResult;
    }

    /**
     * retourne un identifiant du paramètre dans le formulaire
     * surchargé dans le cas des ouvrages //
     */
    protected getParameterRefid(p: ParamDefinition) {
        return p.symbol;
    }

    /**
     * Copies current Nub result into result components for display on page.
     * Should be called every time the Nub result changes.
     * Must be idempotent.
     */
    protected abstract reaffectResultComponents();

    /**
     * Lance le calcul d'un paramètre en déterminant une valeur initiale.
     * Si nécessaire déclenche un calcul en chaîne des modules en amont.
     */
    protected runNubCalc(nub: Nub): Result {
        return nub.CalcSerie();
    }

    protected getComputedParameter(): NgParameter {
        const cpd = this._formBase.currentNub.calculatedParam;
        let ngparam: NgParameter;
        if (cpd !== undefined) {
            ngparam = this._formBase.getParamFromSymbol(cpd.symbol);
            if (ngparam === undefined) { // calculated parameter is not displayed on screen
                ngparam = new NgParameter(cpd, this._formBase);
            }
        }
        return ngparam;
    }

    protected getVariatedParameters(): NgParameter[] {
        let res: NgParameter[] = [];
        // find variated local parameters
        res = this._formBase.getDisplayedParamListFromState(ParamRadioConfig.VAR);
        // add variated linked parameters
        const pms = this._formBase.getDisplayedParamListFromState(ParamRadioConfig.LINK);
        for (const p of pms) {
            if (p.paramDefinition.hasMultipleValues) {
                res.push(p);
            }
        }
        return res;
    }

    /**
     * Triggers computation of the Nub, updates form results
     */
    public doCompute() {
        this._formResult.resetResults();

        this.compute();

        this._formBase.notifyObservers({
            "action": "resultsUpdated",
        }, this._formBase);
    }

    // interface Observer

    public update(sender: any, data: any): void {
        if (sender instanceof Nub) {
            switch (data.action) {
                case "nubResultUpdated":
                    // forward Nub results update notification to FormCompute objects
                    this.reaffectResultComponents();
                    break;
            }
        }
    }
}
