import { SelectFieldReference } from "./select-field-reference";
import { SelectEntry } from "./select-entry";
import { ServiceFactory } from "../services/service-factory";
import { decodeHtml } from "../util";

import { Session, Solveur } from "jalhyd";

/**
 * A select field that populates itself with references to Nubs
 */
export class SelectFieldNub extends SelectFieldReference {

    protected initSelectedValue() {
        const nub = this.parentForm.currentNub;
        if (nub instanceof Solveur) {
            const ntc = nub.nubToCalculate;
            if (ntc !== undefined) {
                this.setValueFromId(this._entriesBaseId + ntc.uid);
            }
        }
    }

    /**
     * Populates entries with available references
     */
    protected populate() {
        switch (this._source) {
            case "solveur_target": // Solveur, paramètre cible (à calculer)
                // find all Nubs having at least one link to another Nub's result
                const fs = ServiceFactory.instance.formulaireService;
                const downstreamNubs = Session.getInstance().getDownstreamNubs();
                for (const dn of downstreamNubs) {
                    const calc = fs.getFormulaireFromId(dn.uid).calculatorName;
                    let label = calc;
                    if (dn.calculatedParam !== undefined) {
                        const varName = fs.expandVariableName(dn.calcType, dn.calculatedParam.symbol);
                        label += ` / ${varName} (${dn.calculatedParam.symbol})`;
                    }
                    this.addEntry(new SelectEntry(this._entriesBaseId + dn.uid, dn.uid, decodeHtml(label)));
                }
                break;
        }
    }

}
